@@ -6,9 +6,10 @@ import re
 import json
 import csv
 import urllib
+import time
 from urllib.parse import urlparse
 from xml.etree import ElementTree as ET
-from typing import List, Dict
+from typing import List, Dict, Optional
 import dateutil.parser
 import requests
 from prettytable import PrettyTable
@@ -28,6 +29,22 @@ class KospexDependencies:
         # The following will be the results from the list of dependencies from the assess command
         self.dependencies = []
 
+    def extract_purl(self, purl):
+        # Extract the purl from the given URL
+        # Example: purl = "pkg:pypi/requests@2.31.0"
+        # Extract the package type, name, and version from the purl
+        # Return a dictionary with the extracted information
+        package_only = purl.split(":")[1]
+        package_type = package_only.split("/")[0]
+        full_package = package_only.split("/")[1]
+        package_name = full_package.split("@")[0]
+        version = full_package.split("@")[1]
+        return {
+            "ecosystem": package_type,
+            "package_name": package_name,
+            "package_version": version
+        }
+
     def deps_dev(self,package_type,package_name,version):
         """ Query the Deps.dev API for a package and version"""
         base_url = "https://api.deps.dev/v3alpha/systems"
@@ -52,21 +69,48 @@ class KospexDependencies:
 
         return data
 
+    def deps_dev_package(self,package_type,package_name):
+        """
+        Query the deps.dev API for a package and get all version history
+        """
+        base_url = "https://api.deps.dev/v3alpha/systems"
+        encoded_name = urllib.parse.quote(package_name, safe='')
+        url = f"{base_url}/{package_type}/packages/{encoded_name}"
+        # links -> which has a SOURCE_REPO label should be the git
+
+        data = None
+
+        if self.kospex_query:
+            content = self.kospex_query.url_request(url)
+            if content:
+                data = json.loads(content)
+        else:
+            req = requests.get(url, timeout=10)
+            if req.status_code == 200:
+                data = req.json()
+            else:
+                print(f"Error: {req.status_code} {req.text}")
+
+        return data
+
+
+
     def get_url_json(self, url, timeout=10, cache=True):
         """ Get the contents of a URL, use the query cache if we can """
         data = None
 
         if self.kospex_query and cache:
-            content = self.kospex_query.url_request(url)
-            data = json.loads(content)
+            if content := self.kospex_query.url_request(url):
+                data = json.loads(content)
         else:
             req = requests.get(url, timeout=timeout)
             if req.status_code == 200:
                 data = req.json()
+
         return data
 
 
-    def get_pypi_package_info(self,package):
+    def get_pypi_package_info(self,package,version: Optional[str] = None):
         """ Get the latest version of a package from PyPI """
         url = f"https://pypi.org/pypi/{package}/json"
 
@@ -113,7 +157,7 @@ class KospexDependencies:
     def is_valid_pypi_package_declaration(self,s):
         """
         Returns True if the string s follows the pattern <package_name>==<version_number>,
-        for packges declared in a requirements.txt (or similar named) file.
+        for packages declared in a requirements.txt (or similar named) file.
         :param s: A string representing the package and version.
         :return: True if the pattern matches, False otherwise.
         """
@@ -123,14 +167,13 @@ class KospexDependencies:
 
     def get_table_field_names(self):
         """ Return the field names for the CSV table """
-        return ["package_name", "package_version", "days_ago",
-                "published_at", "advisories", "default", "versions_behind", "source_repo", "authors"]
+        return ["package_name", "package_version", "versions_behind", "days_ago",
+                "published_at", "default", "advisories", "malware",  "source_repo", "authors"]
 
     def get_cli_pretty_table(self):
         """ Return a pretty table for the CLI """
         table = PrettyTable()
-        #table.field_names = ["package", "version", "days_ago",
-        #                "published_at", "source_repo", "advisories", "default"]
+
         table.field_names = self.get_table_field_names()
         table.align["package_name"] = "l"
         table.align["package_version"] = "r"
@@ -138,9 +181,11 @@ class KospexDependencies:
         table.align["published_at"] = "l"
         table.align["source_repo"] = "l"
         table.align["advisories"] = "r"
+        table.align["malware"] = "r"
         table.align["default"] = "r"
         table.align["versions_behind"] = "r"
         table.align["authors"] = "r"
+
         return table
 
     def is_npm_package(self,filename):
@@ -169,7 +214,7 @@ class KospexDependencies:
         filename (str): The filename to check.
 
         Returns:
-        bool: True if it looks like a .csproj variant, packages.config, etcFalse otherwise.
+        bool: True if it looks like a .csproj variant, packages.config, etc False otherwise.
         """
         # Regular expression to match package.json and its variants
         basefile = os.path.basename(filename)
@@ -257,9 +302,14 @@ class KospexDependencies:
         else:
             print(f"Unknown or unsupported package manager file found {basefile}")
 
+        if results:
+            for dep in results:
+                if publishedAt := dep.get('published_at', None):
+                    dep["days_ago"] = KospexUtils.days_ago(publishedAt)
 
         if save and git_details:
-            print("Should be saving now!")
+            import pprint
+            pprint.PrettyPrinter(indent=4).pprint(results)
             # TODO - see if there is better way of excluding this key
             for r in results:
                 r.pop("days_ago",None)
@@ -378,11 +428,13 @@ class KospexDependencies:
         details = {}
         details['package_name'] = package_name
         details['package_version'] = package_version
+        details['package_type'] = package_type
 
         today = datetime.datetime.now(datetime.timezone.utc)
         # TODO - Handle bad package names
         # TODO - Handle 404 errors (most likely due to bad package name)
-        deps_info = self.deps_dev(package_type,package_name,package_version)
+        if package_version:
+            deps_info = self.deps_dev(package_type,package_name,package_version)
 
         # If we don't get any info back, we'll just return an empty dictionary
         if not deps_info:
@@ -406,8 +458,9 @@ class KospexDependencies:
         details["advisories"] = self.get_advisories_count(deps_info)
 
         # Get the versions behind info
-        days_info = self.get_versions_behind(package_type,package_name,package_version)
-        details['versions_behind'] = days_info.get("versions_behind","Unknown")
+        if package_version:
+            days_info = self.get_versions_behind(package_type,package_name,package_version)
+            details['versions_behind'] = days_info.get("versions_behind","Unknown")
 
         # TODO - this is a hacky way of duplicating the keys needed
         #details['package_name'] = package_name
@@ -473,8 +526,8 @@ class KospexDependencies:
 
         data = None
         if self.kospex_query:
-            content = self.kospex_query.url_request(url)
-            data = json.loads(content)
+            if content := self.kospex_query.url_request(url):
+                data = json.loads(content)
         else:
             req = requests.get(url, timeout=10)
             if req.status_code == 200:
@@ -568,13 +621,13 @@ class KospexDependencies:
                     repo_path = self.extract_repo_path(url) if url else None
                     if url:
                         if repo_path:
-                            row['package'] = repo_path
+                            row['package_name'] = repo_path
                         else:
-                            row['package'] = repo_path
+                            row['package_name'] = repo_path
                     else:
-                        row['package'] = line.strip()
+                        row['package_name'] = line.strip()
 
-                    row["version"] = "Unknown"
+                    row["package_version"] = "Unknown"
                     row["days_ago"] = "Unknown"
                     row["published_at"] = "Unknown"
                     row["source_repo"] = url
@@ -647,6 +700,16 @@ class KospexDependencies:
 
         return records
 
+    def print_dependencies_table(self, records,malware=None):
+        table = self.get_cli_pretty_table()
+        table_rows = []
+
+        for dep in records:
+            table_rows.append(self.get_values_array(dep, self.get_table_field_names(), '-'))
+
+        table.add_rows(table_rows)
+        print(table)
+
     def get_repo_authors(self, repo_url):
         """ Return the number of unique authors for a given repo that's been sync'ed """
         authors = 0
@@ -698,14 +761,14 @@ class KospexDependencies:
         """ Find all dependency files (package managers) in a directory and its subdirectories."""
         # Map of filename to its package manager
         package_files = {
-            'requirements.*\.txt$': 'PyPi',
+            'requirements.*.txt$': 'PyPi',
             'Pipfile': 'PyPi (Pipenv)',
             'Pipfile.lock': 'PyPi (Pipenv)',
             'setup.py': 'PyPi',
             'pyproject.toml': 'PyPi (Poetry/Flit/etc.)',
             'Gemfile': 'RubyGems',
             'Gemfile.lock': 'RubyGems',
-            'package.*\.json': 'npm',
+            'package.*.json': 'npm',
             'yarn.lock': 'Yarn',
             'composer.json': 'Composer',
             'composer.lock': 'Composer',
@@ -798,7 +861,7 @@ class KospexDependencies:
 
         data = self.get_url_json(url)
         versions = data.get('versions', [])
-        # We will need to sort the list, since th   e API returns the versions in a weird order
+        # We will need to sort the list, since the API returns the versions in a weird order
         #print(json.dumps(versions, indent=2))
         #print(len(versions))
 
@@ -938,3 +1001,95 @@ class KospexDependencies:
 
         # Return the results array
         return results
+
+    def check_malware(self, package_type,package_name, api_key):
+        # Beta implementation of malicious packages API
+        url = f"https://api.maliciouspackages.com/package/{package_type}/{package_name}"
+        is_malware = False
+        headers = {
+                    "X-API-Key": api_key
+                }
+        try:
+            time.sleep(1)
+            response = requests.get(url, headers=headers)
+            if response.status_code == 200:
+                malware_result = response.json()
+                is_malware = malware_result.get("malware", False)
+                #result['malware'] = malware_result.get("malware", False)
+            else:
+                print(f"Error checking {package_name}: HTTP {response.status_code}")
+                #result['malware'] = {"error": f"HTTP {response.status_code}"}
+        except Exception as e:
+            print(f"Exception when checking {package_name}: {str(e)}")
+                #result['malware'] = {"error": str(e)}
+        # else:
+        #     print(f"Skipping package with missing type or name: {package_type} and {package_name}")
+
+        return is_malware
+
+    def package_dependencies(self, package: str, version: str, ecosystem: str):
+        """
+        Lookup the package dependencies on deps.dev
+
+        Args:
+            package: Package name to lookup.
+            version: Package version to lookup.
+            ecosystem: Package ecosystem to lookup.
+        """
+        package = package
+        version = version
+        ecosystem = ecosystem
+        #GET /v3/systems/{versionKey.system}/packages/{versionKey.name}/versions/{versionKey.version}:dependencies
+        base_url = "https://api.deps.dev/v3/systems"
+        encoded_name = urllib.parse.quote(package, safe='')
+        url = f"{base_url}/{ecosystem}/packages/{encoded_name}/versions/{version}:dependencies"
+        #https://api.deps.dev/v3alpha/systems/pypi/packages/requests/versions/2.31.0
+        # links -> which has a SOURCE_REPO label should be the git
+
+        print(url)
+
+        data = self.get_url_json(url)
+        # req = requests.get(url, timeout=10)
+        # if req.status_code == 200:
+        #     data = req.json()
+        # # Need to handle HTTP error codes
+
+        if data is None:
+            return None
+
+        nodes = data.get("nodes")
+        for node in nodes:
+
+            node["id"] = self.versionKey_id(node["versionKey"])
+
+            system = node["versionKey"].get("system").lower()
+
+            name = node["versionKey"].get("name").lower()
+            node["name"] = name
+
+            version = node["versionKey"].get("version").lower()
+            node["version"] = version
+
+            dep = self.deps_dev(system, name, version)
+            node["publishedAt"] = dep.get("publishedAt")
+
+            advisories = dep.get("advisoryKeys")
+            num_advisories = len(advisories) if advisories else 0
+            node["advisories"] = num_advisories
+
+            versions = self.get_versions_behind(system, name, version)
+            if versions:
+                node["versions_behind"] = versions.get("versions_behind")
+
+
+        return data
+
+    def versionKey_id(self,versionKey):
+        """
+        Convert a deps.dev version key to a string
+        """
+        system = versionKey.get("system").lower()
+        name = versionKey.get("name").lower()
+        version = versionKey.get("version").lower()
+
+        return ":".join([system, name, version])
