@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 """This is the kospex runner tool - run the same command on all repos."""
+from itertools import count
 import os
 import os.path
 import subprocess
@@ -7,6 +8,7 @@ import sys
 import shlex
 import glob
 import click
+from pydantic.fields import Deprecated
 from kospex_core import Kospex
 from kospex_observation import Observation
 import kospex_utils as KospexUtils
@@ -18,7 +20,6 @@ import kospex_web as KospexWeb
 from rich.console import Console
 from rich.table import Table
 
-
 # Initialize Kospex environment with logging
 KospexUtils.init(create_directories=True, setup_logging=True, verbose=False)
 kospex = Kospex()
@@ -175,7 +176,7 @@ def repo_size(save,csv,request_id,verbose):
     results = []
 
     for r in repos:
-        console.log(f"\n{r['_repo_id']}\t{r['file_path']}")
+        console.log(f"\nCalculating {r['_repo_id']} ...\n{r['file_path']}")
 
         kgit = KospexGit()
         kgit.set_repo(r['file_path'])
@@ -208,6 +209,19 @@ def repo_size(save,csv,request_id,verbose):
             else:
                 kospex.kospex_query.add_observation(obs.to_dict())
 
+    table = Table(title="Repo Size")
+    table.add_column("Repo ID", justify="left", style="cyan", no_wrap=True)
+    table.add_column("total (Mb)", style="magenta",justify="right")
+    table.add_column("total (kb)", style="magenta",justify="right")
+    table.add_column(".git (kb)", style="magenta",justify="right")
+    table.add_column("workspace (kb)", style="magenta",justify="right")
+
+    for r in results:
+        table.add_row( r['repo_id'], str( round( r['total_size'] / 1024, 3)),
+            str(r['total_size']), str(r['git_size']), str(r['workspace_size']) )
+
+    console.print(table)
+
     if csv:
         filename = 'repo-sizes.csv'
         console.log(f"Writing {len(results)} repo sizes to {filename}")
@@ -287,6 +301,281 @@ def developer_tech(csv,verbose,developers,dev):
          console.log(f"Writing {dev} technology to {filename}")
          KrunnerUtils.write_dict_to_csv(filename, results)
 
+@cli.command("dependencies")
+@click.option('-csv', is_flag=True, default=False, help="Save to CSV file. (Default: False)")
+def dependencies(csv):
+    """
+    Show a list of all dependency files
+    use -csv to write the results to the csv file dependencies-list.csv
+    """
+    dependencies = []
+    results = []
+    # This will be the memory kospex query object
+    memory_kq = None
+
+    console.log("Loading data to in memory database ...")
+    with KospexTimer("Loading data to in memory database") as load_memory:
+        memory_kq = kospex.kospex_query.create_memory_kospex_query(
+            ["commit_files", "file_metadata", "repos"])
+    console.log(f"Loaded tables to memory db {load_memory}")
+
+    console.log("Creating indexes ...")
+    with KospexTimer("creating indexes") as index_timer:
+        #memory_kq.kospex_db["commits"].create_index(['hash'])
+        #memory_kq.kospex_db["commit_files"].create_index(['hash'])
+        memory_kq.kospex_db["commit_files"].create_index(['committer_when'])
+    console.log(f"{index_timer}")
+
+    with KospexTimer("Grabbing all dependencies from memory DB") as memory_timer:
+        dependencies = memory_kq.get_dependency_files()
+        #results = memory_kq.developer_tech(author_email=dev,developers=developers)
+    console.log(f"{memory_timer}")
+
+    console.log(f"Finding dependencies ... {len(dependencies)}")
+    counter = 0
+    for d in dependencies:
+        counter += 1
+        console.log(f"Processing dependency {counter}/{len(dependencies)}")
+        console.log(f"repo_id: {d['_repo_id']}")
+        console.log(f"{d['Provider']}\n")
+
+        details = memory_kq.get_last_commit_file(d['_repo_id'], d['Provider'])
+        repo = memory_kq.get_repo_by_id(d['_repo_id'])
+        if repo:
+            d['repo_status'] = KospexUtils.development_status(repo['last_seen'])
+            d['last_repo_commit'] = repo.get('last_seen',"Unknown")
+        else:
+            d['repo_status'] = "Unknown"
+            d['last_repo_commit'] = "Unknown"
+        if details:
+            if not d['committer_when']:
+                d['committer_when'] = details['committer_when']
+        if d['committer_when']:
+            d['status'] = KospexUtils.development_status(d['committer_when'])
+        else:
+            d['status'] = "Unknown"
+
+        dep = {
+            "status": d['status'],
+            "file_path": d['Provider'],
+            "committer_when": d['committer_when'],
+            "repos_status": d['repo_status'],
+            "last_repo_commit": d['last_repo_commit'],
+            "repo_id": d['_repo_id'],
+            "tags": d['tech_type']
+        }
+
+        results.append(dep)
+
+    table = Table(title="Dependencies")
+    table.add_column("Repo ID", justify="left", style="bright_black", no_wrap=True)
+    table.add_column("File Path", style="magenta",justify="left")
+    table.add_column("Last Commit", style="bright_black",justify="right")
+    table.add_column("Status", style="magenta", justify="right")
+
+    for d in results:
+        table.add_row(d['repo_id'], str(d['file_path']),
+            d.get('committer_when',"Unknown"), d['status'])
+
+    console.print(table)
+
+    tech_number = len(dependencies)
+
+    console.log(f"# Dependencies found: {tech_number}")
+
+    if not csv:
+        console.log("\nWarning: No results written to CSV", style="red")
+        console.log("use -csv to export\n", style="red")
+
+    if csv:
+         filename = 'dependencies-list.csv'
+         console.log(f"Writing dependencies to {filename}")
+         KrunnerUtils.write_dict_to_csv(filename, results)
+
+@cli.command("devs-by-tag")
+@click.option('-csv', is_flag=True, default=False, help="Save to CSV file. (Default: False)")
+@click.option('-tag', required=False, type=click.STRING)
+@click.option('-filename', required=False, type=click.STRING)
+def devs_by_tag(csv, tag, filename):
+    """
+    Find all files (last modified only) which have the tag,
+    and the developers that have modified them.
+    use -csv to write the results to the csv file dependencies-list.csv
+    """
+    files = []
+    results = []
+    # This will be the memory kospex query object
+    memory_kq = None
+
+    if not tag and not filename:
+        console.log("\nWarning: One of tag or filename are required", style="red")
+        exit(1)
+
+    console.log("Loading data to in memory database ...")
+    with KospexTimer("Loading data to in memory database") as load_memory:
+        memory_kq = kospex.kospex_query.create_memory_kospex_query(
+            ["commits", "commit_files", "file_metadata"])
+    console.log(f"Loaded tables to memory db {load_memory}")
+
+    console.log("Creating indexes ...")
+    with KospexTimer("creating indexes") as index_timer:
+        memory_kq.kospex_db["commits"].create_index(['hash'])
+        memory_kq.kospex_db["commit_files"].create_index(['hash', 'committer_when'])
+        #memory_kq.kospex_db["commit_files"].create_index(['committer_when'])
+    console.log(f"{index_timer}")
+
+    with KospexTimer("Grabbing all files with the tag {tag} from memory DB") as memory_timer:
+        print(f"tag: {tag}, filename: {filename}")
+        #results = memory_kq.get_developers_by_tag(tag=tag,filename=filename)
+        files = memory_kq.get_metadata_files(tag=tag, filename=filename)
+    console.log(f"{memory_timer}")
+
+    console.log(f"Processing files ... {len(files)}")
+    counter = 0
+    for item in files:
+        counter += 1
+        console.log(f"Processing file {counter}/{len(files)}")
+        console.log(f"repo_id: {item['_repo_id']}")
+        console.log(f"{item['Provider']}\n")
+        authors = memory_kq.get_file_authors(file_name=item['Provider'], repo_id=item['_repo_id'])
+        for author in authors:
+            results.append(author)
+        #console.log(item)
+        #console.log(authors)
+
+    table = Table(title="Authors")
+    table.add_column("Author", style="bright_black",justify="left")
+    table.add_column("File Path", style="magenta",justify="left")
+    table.add_column("Last Commit", style="bright_black",justify="right")
+    table.add_column("Commits", style="blue", justify="right")
+    table.add_column("Repo ID", justify="left", style="bright_black", no_wrap=True)
+
+    for d in results:
+         table.add_row(d['author_email'], str(d['file_path']),
+             d.get('committer_when',"Unknown"), str(d['commits']), d['_repo_id'])
+
+    console.print(table)
+
+    tech_number = len(results)
+
+    console.log(f"# authors and files found: {tech_number}")
+
+    if not csv:
+        console.log("\nWarning: No results written to CSV", style="red")
+        console.log("use -csv to export\n", style="red")
+
+    if csv:
+         filename = 'devs-by-tag.csv'
+         console.log(f"Writing dependencies to {filename}")
+         KrunnerUtils.write_dict_to_csv(filename, results)
+
+@cli.command("tenure")
+@click.option('-days', is_flag=False, default=90, help="Days since last commit = Left (Default: 90)")
+@click.option('-seen', is_flag=False, default=365, help="Seen in the last X days (Default: 365)")
+@click.option('-csv', is_flag=True, default=False, help="Save to CSV file. (Default: False)")
+def tenure(days, seen, csv):
+    """
+    Show the tenure of developers
+    """
+
+    results = []
+
+    # This will be the memory kospex query object
+    memory_kq = None
+
+    console.log("Loading data to in memory database ...")
+    with KospexTimer("Loading data to in memory database") as load_memory:
+        memory_kq = kospex.kospex_query.create_memory_kospex_query(["commits"])
+    console.log(f"Loaded tables to memory db {load_memory}")
+
+    console.log("Creating indexes ...")
+    with KospexTimer("creating indexes") as index_timer:
+        memory_kq.kospex_db["commits"].create_index(['committer_when'])
+    console.log(f"{index_timer}")
+
+    console.log("Creating indexes ...")
+    with KospexTimer("Calculating tenure") as memory_timer:
+        results = memory_kq.authors()
+    console.log(f"{memory_timer}")
+
+    # email, first_commit, last_commit, years_active, repos
+
+    table = Table(title="Authors")
+    table.add_column("Developer", style="bright_black",justify="left")
+    table.add_column("First Commit", style="magenta",justify="left")
+    table.add_column("Last Commit", style="bright_black",justify="right")
+    table.add_column("Years Active", style="blue", justify="right")
+    table.add_column("# Repos", justify="right", style="bright_black")
+    table.add_column("# Commits", justify="right", style="bright_black")
+
+    tenure_groups = {
+        "< 1 year": 0,
+        "1-2 years": 0,
+        "2-3 years": 0,
+        "3-4 years": 0,
+        "4-5 years": 0,
+        "5+ years": 0
+    }
+
+    leavers = 0
+
+    developer_leavers = []
+
+    for dev in results:
+
+        # If last_seen > days they've left
+        # and we've seen them in the seen timeframe (default we've seen them in the last year)
+        #
+        if dev['last_seen'] > days and seen > dev['last_seen']:
+            leavers += 1
+            days_active = KospexUtils.days_between_datetimes(dev['first_commit'], dev['last_commit'])
+            years_active = round(days_active / 365,3)
+            dev['years_active'] = years_active
+
+            if years_active < 1:
+                tenure_groups["< 1 year"] += 1
+            elif years_active < 2:
+                tenure_groups["1-2 years"] += 1
+            elif years_active < 3:
+                tenure_groups["2-3 years"] += 1
+            elif years_active < 4:
+                tenure_groups["3-4 years"] += 1
+            elif years_active < 5:
+                tenure_groups["4-5 years"] += 1
+            else:
+                tenure_groups["5+ years"] += 1
+
+            table.add_row(dev['author_email'], dev['first_commit'], dev['last_commit'],
+                str(years_active), str(dev['repos']), str(dev['commits']))
+
+            developer_leavers.append(dev)
+
+    console.print(table)
+
+    tenure_percentages = KospexUtils.convert_to_percentage(tenure_groups)
+
+    stat_table = Table(title="Tenure of Leavers Summary")
+    stat_table.add_column("Tenure", style="bright_black",justify="left")
+    stat_table.add_column("Number", style="bright_black",justify="right")
+    stat_table.add_column("Percentage", style="bright_black",justify="right")
+    for k in tenure_percentages.keys():
+        stat_table.add_row(k, str(tenure_groups[k]), str(tenure_percentages[k])+"%")
+
+    console.print()
+    console.print(stat_table)
+    console.print(f"\nTotal number of developers seen: {len(results)}")
+    console.print(f"Number of Leavers: {leavers}\n")
+
+    if not csv:
+        console.print("\nWarning: No results written to CSV", style="red")
+        console.print("use -csv to export\n", style="red")
+
+    if csv:
+          filename = 'developer-tenure-leavers.csv'
+          console.log(f"Writing dependencies to {filename}")
+          KrunnerUtils.write_dict_to_csv(filename, developer_leavers)
+
+
 @cli.command("find-docker")
 @click.option('-out', type=click.STRING, help="filename to write CSV results to.")
 @click.option('-images', is_flag=True, default=False, help="Extract image names")
